
# q-expansion of newform 155.3.c.b, downloaded from the LMFDB on 18 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 155
weight = 3
poly_data = [-1, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[-1, 2], 1]]

hecke_ring_character_values = [[32, [-1, 0]], [96, [-1, 0]]]
aps_data = [[0, 0], [0, -1], [5, 0], [0, 0], [0, 0], [0, 6], [0, 15], [7, 0], [0, -18], [0, 0], [-31, 0], [0, -33], [-73, 0], [0, 27], [0, 0], [0, 3], [-37, 0], [0, 0], [0, 0], [-137, 0], [0, -45], [0, 0], [0, -69], [0, 0], [0, 0], [-77, 0], [0, 0], [0, 0], [187, 0], [0, 0], [0, 30], [-17, 0], [0, 99], [0, 0], [143, 0], [0, 0], [0, 0], [0, 0], [0, 27], [0, 0], [0, 0], [0, 0], [-238, 0], [0, 0], [0, -138], [0, 0], [298, 0], [0, -33], [0, 0], [0, 0], [0, 0], [0, 0], [0, 0], [0, 0], [0, 0], [0, -129], [0, 0], [0, 0], [0, -237], [283, 0], [0, 0], [0, 0], [0, 0], [467, 0], [0, 123], [0, 0], [0, 0], [0, -9], [0, 246], [-697, 0], [0, -66], [-398, 0], [0, 291], [0, 0], [-637, 0], [0, 327], [0, 0], [0, 0], [0, 0], [0, 0], [683, 0], [-553, 0], [583, 0], [0, -258], [-517, 0], [0, 0], [0, 0], [0, -18], [0, 0], [0, 414], [0, 0], [338, 0], [0, 375], [0, 0], [0, 0], [0, 0], [0, 0], [763, 0], [0, -381], [-902, 0], [0, 0], [0, 486], [0, 0], [0, 0], [0, 0], [0, 0], [0, -330], [0, 0], [82, 0], [0, 0], [0, 0], [0, 519], [0, 0], [0, 0], [0, 0], [0, 0], [0, 75], [0, 363], [0, 0], [-1193, 0], [-197, 0], [0, -369], [0, -81], [0, 0], [-13, 0], [-1078, 0], [0, 0], [0, 0], [0, 0], [0, 0], [0, 0], [0, 579], [1378, 0], [0, 99], [0, 0], [143, 0], [0, -330], [0, 678], [0, -90], [0, 0], [227, 0], [0, 0], [0, -729], [0, 303], [0, 0], [-833, 0], [0, 0], [0, 0], [0, 0], [0, -453], [0, 0], [0, 0], [0, 615], [0, 0], [0, 0], [0, 0], [1807, 0], [0, 0], [0, 0], [0, 0], [0, 759], [0, 75], [0, -498], [-1933, 0], [0, 0], [0, 654], [0, 0], [0, 0]]
